import graphene
from graphql_jwt.decorators import login_required, permission_required
from graphene_django_extras import DjangoFilterPaginateListField, LimitOffsetGraphqlPagination

from django.utils import timezone
from django.db.models import Sum, Subquery
from django.db.models.functions import Coalesce
from django.contrib.auth import get_user_model

from allianceauth.authentication.models import CharacterOwnership
from allianceauth.eveonline.models import EveCharacter

from allianceauth_pve.models import Rotation, EntryCharacter
from allianceauth_graphql.eveonline.types import EveCharacterType

from .types import RotationType, EntryType, RattingSummaryType


User = get_user_model()


class Query:
    rotation = graphene.Field(RotationType, id=graphene.Int(required=True))
    closed_rotations = graphene.List(RotationType)
    char_running_averages = graphene.Field(RattingSummaryType, start_date=graphene.Date(required=True), end_date=graphene.Date())
    active_rotations = graphene.List(RotationType)
    rotation_entries = DjangoFilterPaginateListField(EntryType, pagination=LimitOffsetGraphqlPagination(), fields=['rotation_id'])
    search_rotation_characters = graphene.List(EveCharacterType, name=graphene.String(required=True))

    @permission_required('allianceauth_pve.view_rotation')
    @login_required
    def resolve_rotation(self, info, id):
        return Rotation.objects.get(pk=id)

    @permission_required('allianceauth_pve.view_rotation')
    @login_required
    def resolve_closed_rotations(self, info):
        return Rotation.objects.filter(is_closed=True).order_by('-closed_at')

    @login_required
    def resolve_char_running_averages(self, info, start_date, end_date=timezone.now().date()):
        char = info.context.user
        rotations = Rotation.objects.filter(closed_at__range=(start_date, end_date)).get_setup_summary().filter(character_id=char).values('total_setups')
        return EntryCharacter.objects.filter(entry__rotation__closed_at__range=(start_date, end_date), character=char)\
            .values('character').order_by()\
            .annotate(helped_setups=Coalesce(Subquery(rotations[:1]), 0))\
            .annotate(estimated_total=Sum('estimated_share_total'))\
            .annotate(actual_total=Sum('actual_share_total'))

    @permission_required('allianceauth_pve.view_rotation')
    @login_required
    def resolve_active_rotations(self, info):
        return Rotation.objects.filter(is_closed=False).order_by('-priority')

    @permission_required('allianceauth_pve.view_rotation')
    @login_required
    def resolve_search_rotation_characters(self, info, name):
        mains = CharacterOwnership.objects.filter(character__character_name__icontains=name).values('user__profile__main_character')
        return EveCharacter.objects.filter(pk__in=mains).distinct()



import graphene
import graphql_jwt
from graphql_jwt.shortcuts import get_token
from graphql_jwt.refresh_token.shortcuts import create_refresh_token
from graphql_jwt.decorators import login_required
from django.contrib.auth import authenticate
from django.urls import reverse
from django.core.mail import send_mail
from django.core import signing
from django.conf import settings
from graphene_django.forms.mutation import DjangoFormMutation

from esi.models import Token
from allianceauth.authentication.models import CharacterOwnership

from .forms import EmailRegistrationForm
from .types import UserProfileType, LoginStatus

REGISTRATION_SALT = getattr(settings, "REGISTRATION_SALT", "registration")


class EsiTokenAuthMutation(graphene.Mutation):
    """Login Mutation

    Receives the esi code from callback and provides a token:
    in case the status is "LOGIN", the token is for the Authorization header (Authorization: JWT <token>);
    in case the status is "REGISTRATION", the token is for the RegistrationMutation mutation argument.
    """
    me = graphene.Field(UserProfileType)
    token = graphene.String()
    refresh_token = graphene.String()
    errors = graphene.List(graphene.String)
    status = graphene.Field(LoginStatus)

    class Arguments:
        sso_token = graphene.String(required=True, description="The code param received from esi callback")

    @classmethod
    def mutate(cls, root, info, sso_token):
        errors = []
        token_obj = Token.objects.create_from_code(sso_token)
        user = authenticate(token=token_obj)
        status = 0

        if user:
            token_obj.user = user
            if Token.objects.exclude(pk=token_obj.pk).equivalent_to(token_obj).require_valid().exists():
                token_obj.delete()
            else:
                token_obj.save()

            if user.is_active:
                status = 1
            elif not user.email:
                status = 2
                info.context.session.update({'registration_uid': user.pk})
                info.context.session.save()
            else:
                errors.append('Unable to authenticate the selected character')

        else:
            errors.append('Unable to authenticate the selected character')

        if status == 1:
            token = get_token(user)
            refresh_token = create_refresh_token(user).get_token()
        elif status == 2:
            refresh_token = None
            token = signing.dumps(user.pk)
        else:
            token = refresh_token = None

        return cls(
            me=user.profile if status == 1 else None,
            token=token,
            refresh_token=refresh_token,
            errors=errors,
            status=status
        )


class RegistrationMutation(DjangoFormMutation):
    """Email registration

    Receives the token from LoginMutation (if the status is "REGISTRATION") and the email and sends an email with the activation link.
    """
    class Meta:
        form_class = EmailRegistrationForm

    ok = graphene.Boolean()
    errors = graphene.List(graphene.String)

    @classmethod
    def perform_mutate(cls, form, info):
        errors = []
        user_id = signing.loads(form.data['token'])
        email = form.data['email']

        site = getattr(settings, 'REDIRECT_SITE')

        activation_key = signing.dumps([user_id, email], salt=REGISTRATION_SALT)

        full_url = info.context.build_absolute_uri(reverse('allianceauth_graphql:verify_email')) + f"?activation_key={activation_key}"

        send_mail(
            f'Account activation for {site}',
            f"""< p >
            You're receiving this email because someone has entered this email address while registering for an account on {site}
            < /p >

            < p >
            If this was you, please click on the link below to confirm your email address:
            < p >

            < p >
            < a href="{full_url}" > Confirm email address < /a >
            < / p >

            < p >
            If this was not you, it is safe to ignore this email.
            < /p >""",
            settings.DEFAULT_FROM_EMAIL,
            [email],
            fail_silently=False
        )

        return cls(ok=True, errors=errors, **form.data)


class ChangeMainCharacterMutation(graphene.Mutation):
    """Mutation for changing main character, assuming this character has already been added and it's not owned by another user
    """
    class Arguments:
        new_main_character_id = graphene.Int(required=True)

    ok = graphene.Boolean()
    errors = graphene.List(graphene.String)
    me = graphene.Field(UserProfileType)

    @classmethod
    @login_required
    def mutate(cls, root, info, new_main_character_id):
        errors = []
        profile = info.context.user.profile
        try:
            co = CharacterOwnership.objects.get(character__character_id=new_main_character_id, user=info.context.user)
            ok = True
        except CharacterOwnership.DoesNotExist:
            ok = False
            if not CharacterOwnership.objects.filter(character__character_id=new_main_character_id).exists():
                errors.append("You never added this character")
            else:
                errors.append("You don't own this character")

        if ok:
            profile.main_character = co.character
            profile.save(update_fields=['main_character'])

        return cls(ok=ok, errors=errors, me=profile)


class AddCharacterMutation(graphene.Mutation):
    """Mutation for adding a new character to the list of alts.
    Receives the esi code from callback of the character to add.
    """
    class Arguments:
        new_char_sso_token = graphene.String(required=True)

    ok = graphene.Boolean()
    errors = graphene.List(graphene.String)
    me = graphene.Field(UserProfileType)

    @classmethod
    @login_required
    def mutate(cls, root, info, new_char_sso_token):
        errors = []
        user = info.context.user
        token_obj = Token.objects.create_from_code(new_char_sso_token, user=user)
        if not CharacterOwnership.objects.filter(user=user, character__character_id=token_obj.character_id, owner_hash=token_obj.character_owner_hash).exists():
            errors.append("This character already has an account")
            ok = False
        else:
            ok = True

        return cls(ok=ok, me=user.profile, errors=errors)


class Mutation:
    token_auth = EsiTokenAuthMutation.Field()
    verify_token = graphql_jwt.Verify.Field()
    refresh_token = graphql_jwt.Refresh.Field()
    revoke_token = graphql_jwt.Revoke.Field()
    change_main_character = ChangeMainCharacterMutation.Field()
    add_character = AddCharacterMutation.Field()
    email_registration = RegistrationMutation.Field()



from django.urls import path
from django.conf import settings
from graphene_django.views import GraphQLView
from django.views.decorators.csrf import csrf_exempt
from .schema import schema
from .authentication.views import verify_email

app_name = 'allianceauth_graphql'


urlpatterns = [
    path("", csrf_exempt(GraphQLView.as_view(graphiql=getattr(settings, 'SHOW_GRAPHIQL', True), schema=schema)), name='graphql'),
    path('verify/', verify_email, name='verify_email')
]



import graphene
from graphene_django import DjangoObjectType
from allianceauth.authentication.models import UserProfile, State
from django.contrib.auth.models import Group


class StateType(DjangoObjectType):
    class Meta:
        model = State
        fields = ('name', )


class UserProfileType(DjangoObjectType):
    class Meta:
        model = UserProfile


class GroupType(DjangoObjectType):
    class Meta:
        model = Group


class LoginStatus(graphene.Enum):
    ERROR = 0
    LOGGED_IN = 1
    REGISTRATION = 2



import graphene


class EntryCharacterInput(graphene.InputObjectType):
    share_count = graphene.Int(required=True)
    character_id = graphene.Int(required=True)
    helped_setup = graphene.Boolean(default_value=False)


class EntryInput(graphene.InputObjectType):
    estimated_total = graphene.Float(required=True)
    shares = graphene.List(EntryCharacterInput, required=True)


class CreateRotationInput(graphene.InputObjectType):
    name = graphene.String(required=True)
    tax_rate = graphene.Float(required=True)
    priority = graphene.Int(default_value=0.0)


class RotationCloseInput(graphene.InputObjectType):
    rotation_id = graphene.Int(required=True)
    sales_value = graphene.Float(required=True)


class CharactersStatsInput(graphene.InputObjectType):
    start_date = graphene.Date(required=True)
    end_date = graphene.Date(required=True)



#!/usr/bin/env python

"""The setup script."""

from setuptools import setup, find_packages

with open('README.md') as readme_file:
    readme = readme_file.read()

requirements = [
    'allianceauth>=2.11.2,<3.0.0',
    'graphene-django>=2.13,<3.0.0',
    'django-graphql-jwt>=0.3.0,<0.4.0',
    # 'django-cors-headers>=3.7,<4.0.0',
]

test_requirements = []

setup(
    author="Matteo Ghia",
    author_email='matteo.ghia@yahoo.it',
    python_requires='>=3.6',
    classifiers=[
        'Environment :: Web Environment',
        'Framework :: Django',
        'Framework :: Django :: 3.2',
        'Operating System :: POSIX :: Linux',
        'Development Status :: 3 - Alpha',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: GNU General Public License v3 (GPLv3)',
        'Natural Language :: English',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: 3.8',
        'Programming Language :: Python :: 3.9',
        'Programming Language :: Python :: 3.10',
        'Topic :: Internet :: WWW/HTTP',
        'Topic :: Internet :: WWW/HTTP :: Dynamic Content',
    ],
    description="GraphQL integration for AllianceAuth",
    install_requires=requirements,
    extras_require={
        'allianceauth-pve': [
            'graphene-django-extras>=0.4.6,<0.5.0',
            'allianceauth-pve>=0.3.0,<0.4.0',
        ]
    },
    license="GNU General Public License v3",
    long_description=readme,
    long_description_content_type='text/markdown',
    include_package_data=True,
    keywords='allianceauth_graphql',
    name='allianceauth_graphql',
    packages=find_packages(include=['allianceauth_graphql', 'allianceauth_graphql.*']),
    test_suite='tests',
    tests_require=test_requirements,
    url='https://github.com/Maestro-Zacht/allianceauth-graphql',
    version='0.5.7',
    zip_safe=False,
)



import graphene
from graphene_django import DjangoObjectType
from django.contrib.auth import get_user_model

from allianceauth.services.hooks import get_extension_logger

from allianceauth_pve.models import Rotation, EntryCharacter, Entry


logger = get_extension_logger(__name__)

User = get_user_model()


class RattingSummaryType(graphene.ObjectType):
    character = graphene.Field('allianceauth_graphql.eveonline.types.EveCharacterType')
    helped_setups = graphene.Int()
    estimated_total = graphene.Float()
    actual_total = graphene.Float()

    def resolve_character(self, info):
        return User.objects.get(pk=self['character']).profile.main_character


class EntryCharacterType(DjangoObjectType):
    class Meta:
        model = EntryCharacter


class EntryType(DjangoObjectType):
    estimated_total_after_tax = graphene.Float()
    total_shares_count = graphene.Int()

    class Meta:
        model = Entry


class RotationType(DjangoObjectType):
    sales_percentage = graphene.Float()
    days_since = graphene.Int()
    estimated_total = graphene.Float()
    summary = graphene.List(RattingSummaryType)

    class Meta:
        model = Rotation

    def resolve_entries(self, info):
        return self.entries.order_by('-created_at')

    def resolve_summary(self, info):
        return self.summary.order_by('-estimated_total')
